@@ -46,7 +46,7 @@ __PACKAGE__->add_columns(
 );
 __PACKAGE__->set_primary_key('dvd_id');
 __PACKAGE__->belongs_to('owner', 'DBSchema::Result::User', 'owner');
-__PACKAGE__->belongs_to('current_borrower', 'DBSchema::Result::User', 'current_borrower');
+__PACKAGE__->belongs_to('current_borrower', 'DBSchema::Result::User', 'current_borrower', { join_type => "LEFT" });
 __PACKAGE__->has_many('dvdtags', 'Dvdtag', { 'foreign.dvd' => 'self.dvd_id' });
 __PACKAGE__->has_many('viewings', 'DBSchema::Result::Viewing', { 'foreign.dvd_id' => 'self.dvd_id' });
 __PACKAGE__->many_to_many('tags', 'dvdtags' => 'tag');
@@ -58,7 +58,11 @@ __PACKAGE__->add_relationship('like_has_many', 'DBSchema::Result::Twokeys', { 'f
 __PACKAGE__->add_relationship('like_has_many2', 'DBSchema::Result::Twokeys_belongsto', 
     { 'foreign.key1' => 'self.twokeysfk' }, 
     { accessor => 'multi' },
-); 
+);
+__PACKAGE__->has_many(
+    keysbymethod => 'KeysByMethod',
+    { 'foreign.dvd' => 'self.dvd_id' }
+);
 
 1;
 
